@@ -326,7 +326,7 @@ Average movie rating
 
         @aggregated('ratings', sa.Column(sa.Numeric))
         def avg_rating(self):
-            return sa.func.sum(Product.price)
+            return sa.func.avg(Rating.stars)
 
         ratings = sa.orm.relationship('Rating')
 
@@ -375,6 +375,9 @@ except ImportError:
     # SQLAlchemy 0.8
     from sqlalchemy.sql.expression import _FunctionGenerator
 
+from .functions.orm import get_column_key
+from .relationships import chained_join, select_aggregate
+
 
 aggregated_attrs = WeakKeyDictionary(defaultdict(list))
 
@@ -402,54 +405,35 @@ class AggregatedAttribute(declared_attr):
         return desc.column
 
 
-def get_aggregate_query(agg_expr, relationships):
-    """
-    Return a subquery for fetching an aggregate value of given aggregate
-    expression and given sequence of relationships.
-
-    The returned aggregate query can be used when updating denormalized column
-    value with query such as:
-
-    UPDATE table SET column = {aggregate_query}
-    WHERE {condition}
-
-    :param agg_expr:
-        an expression to be selected, for example sa.func.count('1')
-    :param relationships:
-        Sequence of relationships to be used for building the aggregate
-        query.
-    """
-    from_ = relationships[0].mapper.class_.__table__
-    for relationship in relationships[0:-1]:
-        property_ = relationship.property
-        if property_.secondary is not None:
-            from_ = from_.join(
-                property_.secondary,
-                property_.secondaryjoin
-            )
-
-        from_ = (
-            from_
-            .join(
-                property_.parent.class_,
-                property_.primaryjoin
-            )
-        )
-
-    prop = relationships[-1].property
-    condition = prop.primaryjoin
+def local_condition(prop, objects):
+    pairs = prop.local_remote_pairs
     if prop.secondary is not None:
-        from_ = from_.join(
-            prop.secondary,
-            prop.secondaryjoin
-        )
+        parent_column = pairs[1][0]
+        fetched_column = pairs[1][0]
+    else:
+        parent_column = pairs[0][0]
+        fetched_column = pairs[0][1]
 
-    query = sa.select(
-        [agg_expr],
-        from_obj=[from_]
-    )
+    key = get_column_key(prop.mapper, fetched_column)
 
-    return query.where(condition)
+    values = []
+    for obj in objects:
+        try:
+            values.append(getattr(obj, key))
+        except sa.orm.exc.ObjectDeletedError:
+            pass
+
+    if values:
+        return parent_column.in_(values)
+
+
+def aggregate_expression(expr, class_):
+    if isinstance(expr, sa.sql.visitors.Visitable):
+        return expr
+    elif isinstance(expr, _FunctionGenerator):
+        return expr(sa.sql.text('1'))
+    else:
+        return expr(class_)
 
 
 class AggregatedValue(object):
@@ -457,19 +441,11 @@ class AggregatedValue(object):
         self.class_ = class_
         self.attr = attr
         self.relationships = relationships
-        self.expr = self.aggregate_expression(expr, class_)
-
-    def aggregate_expression(self, expr, class_):
-        if isinstance(expr, sa.sql.visitors.Visitable):
-            return expr
-        elif isinstance(expr, _FunctionGenerator):
-            return expr(sa.sql.text('1'))
-        else:
-            return expr(class_)
+        self.expr = aggregate_expression(expr, class_)
 
     @property
     def aggregate_query(self):
-        query = get_aggregate_query(self.expr, self.relationships)
+        query = select_aggregate(self.expr, self.relationships)
 
         return query.correlate(self.class_).as_scalar()
 
@@ -480,7 +456,7 @@ class AggregatedValue(object):
         )
         if len(self.relationships) == 1:
             prop = self.relationships[-1].property
-            condition = self.local_condition(prop, objects)
+            condition = local_condition(prop, objects)
             if condition is not None:
                 return query.where(condition)
         else:
@@ -498,7 +474,7 @@ class AggregatedValue(object):
             remote_pairs = property_.local_remote_pairs
             local = remote_pairs[0][0]
             remote = remote_pairs[0][1]
-            condition = self.local_condition(
+            condition = local_condition(
                 self.relationships[0].property,
                 objects
             )
@@ -507,55 +483,15 @@ class AggregatedValue(object):
                     local.in_(
                         sa.select(
                             [remote],
-                            from_obj=[self.multi_level_aggregate_query_base]
+                            from_obj=[
+                                chained_join(*reversed(self.relationships))
+                            ]
                         ).where(
                             condition
                         )
                     )
                 )
 
-    @property
-    def multi_level_aggregate_query_base(self):
-        property_ = self.relationships[-1].property
-
-        from_ = property_.mapper.class_.__table__
-        for relationship in reversed(self.relationships[0:-1]):
-            property_ = relationship.property
-            if property_.secondary is not None:
-                from_ = from_.join(
-                    property_.secondary,
-                    property_.primaryjoin
-                )
-                from_ = from_.join(
-                    property_.mapper.class_,
-                    property_.secondaryjoin
-                )
-            else:
-                from_ = from_.join(
-                    property_.mapper.class_,
-                    property_.primaryjoin
-                )
-        return from_
-
-    def local_condition(self, prop, objects):
-        pairs = prop.local_remote_pairs
-        if prop.secondary is not None:
-            column = pairs[1][0]
-            key = pairs[1][0].key
-        else:
-            column = pairs[0][0]
-            key = pairs[0][1].key
-
-        values = []
-        for obj in objects:
-            try:
-                values.append(getattr(obj, key))
-            except sa.orm.exc.ObjectDeletedError:
-                pass
-
-        if values:
-            return column.in_(values)
-
 
 class AggregationManager(object):
     def __init__(self):
