@@ -1,5 +1,11 @@
+babel = None
+try:
+    import babel
+except ImportError:
+    pass
 import six
 from sqlalchemy import types
+from sqlalchemy_utils.exceptions import ImproperlyConfigured
 from sqlalchemy_utils.primitives import WeekDay, WeekDays
 from .scalar_coercible import ScalarCoercible
 from .bit import BitType
@@ -48,6 +54,14 @@ class WeekDaysType(types.TypeDecorator, ScalarCoercible):
 
     impl = BitType(WeekDay.NUM_WEEK_DAYS)
 
+    def __init__(self, *args, **kwargs):
+        if babel is None:
+            raise ImproperlyConfigured(
+                "'babel' package is required to use 'WeekDaysType'"
+            )
+
+        super(WeekDaysType, self).__init__(*args, **kwargs)
+
     @property
     def comparator_factory(self):
         return self.impl.comparator_factory
